@@ -15,7 +15,6 @@ from .typing_ import Tensorlike, ListOrSlice, IntOrSlice
 from .grid import Grid
 from .backend import backend as bd
 
-
 ## Boundary Conditions [base class]
 class Boundary:
     """ an FDTD Boundary [base class] """
@@ -50,6 +49,8 @@ class Boundary:
         self.x = self._handle_slice(x)
         self.y = self._handle_slice(y)
         self.z = self._handle_slice(z)
+
+        #Put into unique_name_check function?
         if self.name is not None:
             if not hasattr(grid, self.name):
                 setattr(grid, self.name, self)
@@ -611,3 +612,34 @@ class _PMLZhigh(PML):
         sigma = self._sigma(bd.arange(1.0, self.thickness, 1.0))
         self.sigmaH = bd.zeros((self.grid.Nx, self.grid.Ny, self.thickness, 3))
         self.sigmaH[:, :, :-1, 2] = sigma[None, None, :]
+
+
+def DomainBorderPML(grid, border_cells=5):
+    '''
+    Some problem setups require a layer of PML all the way around the problem.
+    This is a convenience function to add such a layer to an existing grid.
+    Caution: Alters grid in-place.
+    '''
+    # Doesn't return anything, unlike other .boundaries functions - is that breaking the API?
+    # There might be a more straightforward numpy slicing solution...
+    # maybe with
+    # raw = np.pad(raw, [(pcb.pml_cells, pcb.pml_cells), (pcb.pml_cells, pcb.pml_cells),
+    #    (pcb.pml_cells, pcb.pml_cells)], mode='constant') and then indexing the grid by that?
+
+    if(grid.Nx < border_cells*2 or grid.Ny < border_cells*2 or grid.Nz < border_cells*2):
+        raise IndexError("PML border_cells larger than domain!")
+
+    #overly-complicated slicing doesn't overlap,
+    #haven't checked if that's important
+
+    #top and bottom
+    grid[:, : ,0:border_cells] = PML()
+    grid[:, : ,-border_cells:] = PML()
+
+    #left and right
+    grid[0:border_cells, :, border_cells:-border_cells] = PML()
+    grid[-border_cells:, :, border_cells:-border_cells] = PML()
+
+    #front and back
+    grid[border_cells:-border_cells, 0:border_cells, border_cells:-border_cells] = PML()
+    grid[border_cells:-border_cells, -border_cells:, border_cells:-border_cells] = PML()
